unit uAusnahmebehandlungDemo;

{ **************************************************************************** }
{ **************************************************************************** }

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons;

type
  TForm1 = class(TForm)
    BtnBerechnen: TButton;
    PnlUeberschrift: TPanel;
    EdtZahl: TEdit;
    BtnBildLaden: TButton;
    ImgBildLaden: TImage;
    BtnUmwandeln: TButton;
    GrpBoxDivisionDurchNull: TGroupBox;
    GrpBoxDateibehandlung: TGroupBox;
    GrpBoxTypkonvertierung: TGroupBox;
    BitBtnSchliessen: TBitBtn;
    BitBtnInformation: TBitBtn;
    RadGrpEHVerfahren: TRadioGroup;
    ShpBildLaden: TShape;
    EdtDividend: TEdit;
    EdtDivisor: TEdit;
    LblDividend: TLabel;
    LblDivisor: TLabel;
    LblZahl: TLabel;
    procedure BtnBerechnenClick(Sender: TObject);
    procedure BtnBildLadenClick(Sender: TObject);
    procedure BtnUmwandelnClick(Sender: TObject);
    procedure BitBtnInformationClick(Sender: TObject);
    procedure BitBtnSchliessenClick(Sender: TObject);
    procedure EdtZahlKeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

  { Eigene aus Exception-Klasse abgeleitete EMyException }
  EMyException = class(Exception);

var
  Form1: TForm1;

{ **************************************************************************** }
{ **************************************************************************** }

implementation

{$R *.dfm}

procedure TForm1.BtnBerechnenClick(Sender: TObject);
var
  Dividend,Divisor,Quotient: Real;
begin
  { Initialisierung }
  Dividend := StrToFloat(EdtDividend.Text);
  Divisor := StrToFloat(EdtDivisor.Text);

  { Berechnung }
  case RadGrpEHVerfahren.ItemIndex of
    0: begin  { "ohne" }
         Quotient := Dividend / Divisor;
         ShowMessage('Quotient: ' + FloatToStr(Quotient));
       end;

    1: { "If-Anweisung" }
       if Divisor = 0 then
         ShowMessage('If-Anweisung' + #13 + #13 + 'Teilen durch Null verboten!')
         else begin
           Quotient := Dividend / Divisor;
           ShowMessage('If-Anweisung' + #13 + #13 + 'Quotient: ' + FloatToStr(Quotient));
         end;

    2: { "Try-Except-Finally" ("Extended Version", mit versch. optionalen Moeglichkeiten) }
       try  { Benoetigt wg. zusaetzlichem finally-Block unten }
         try
           Quotient := Dividend / Divisor;
           ShowMessage('Try-Except-Finally' + #13 + #13 + 'Quotient: ' + FloatToStr(Quotient));
         except
           on E:EZeroDivide do begin
             ShowMessage('Try-Except-Finally' + #13 + #13 + 'Teilen durch Null verboten!');
             { ShowMessage(E.Message);  // Ausgabe der Nachricht aus Exception }
             { raise EMyException.Create('Try-Except-Finally' + #13 + #13 + 'Teilen durch Null verboten!');  // Ausgabe einer selbst definierten Exception }
             { raise;  // Ausgabe der Standard-Exception-Fehlermeldung }
           end;
         end;  // inneres try
       finally
         { Weitere moegliche Anweisungen }
       end;  // aeusseres try

       { Auch moeglich sind mehrere except-Abfragen zusammen, z.B.:
         except
           on E:EZeroDivide do ShowMessage('Division durch Null!');
           on E: EConvertError do ShowMessage('Konvertierungsfehler!'); }

       { Weitere moegliche Exceptions siehe Unit SysUtils, bspw. in der Deplhi-Hilfe }
  end;  // case
end;

{ **************************************************************************** }

procedure TForm1.BtnUmwandelnClick(Sender: TObject);
var
  Zahl,i: Integer;
  ZahlUngueltig: Boolean;
begin
  case RadGrpEHVerfahren.ItemIndex of
    0: begin  { "ohne" }
         Zahl := StrToInt(EdtZahl.Text);
         ShowMessage('Zahl: ' + IntToStr(Zahl));
       end;

    1: begin  { "If-Anweisung" }
         ZahlUngueltig := False;
         for i := 1 to Length(EdtZahl.Text) do
           if not (EdtZahl.Text[i] in ['0'..'9']) then ZahlUngueltig := True;
         if ZahlUngueltig then
           ShowMessage('If-Anweisung' + #13 + #13 + 'Keine gltige Ganzzahl!')
         else begin
           Zahl := StrToInt(EdtZahl.Text);
           ShowMessage('If-Anweisung' + #13 + #13 + 'Zahl: ' + IntToStr(Zahl));
         end;
       end;
       { Alternativ: Ungueltige Zeichen mit
         if not (Key in ['0'..'9',#8]) then Key := #0;
         in EdtZahlKeyPress abfangen (s.u.)! }

    2: try { "Try-Except-Finally" (hier moeglichst einfach gehalten) }
         Zahl := StrToInt(EdtZahl.Text);
         ShowMessage('Try-Except-Finally' + #13 + #13 + 'Zahl: ' + IntToStr(Zahl));
       except
         on E:EConvertError do ShowMessage('Try-Except-Finally' + #13 + #13 + 'Keine gltige Ganzzahl!');
       end;
       { Alternativ: Verwendung von TryStrToInt und TryStrToFloat, z.B.:
         if not(TryStrToInt(EdtZahl.Text,Zahl)) then ShowMessage('Error!')
           else ShowMessage('Klappt!'); }
  end;  // case
end;

{ **************************************************************************** }

procedure TForm1.BtnBildLadenClick(Sender: TObject);
begin
  case RadGrpEHVerfahren.ItemIndex of
    0: { "ohne" }
       ImgBildLaden.Picture.LoadFromFile('MiJoZi.bmp');

    1: begin  { "If-Anweisung" }
         if FileExists('MiJoZi.bmp') then ImgBildLaden.Picture.LoadFromFile('MiJoZi.bmp')
           else ShowMessage('If-Anweisung' + #13 + #13 + 'Datei nicht gefunden!');
       end;

    2: try  { "Try-Except-Finally" (hier moeglichst einfach gehalten) }
         ImgBildLaden.Picture.LoadFromFile('MiJoZi.bmp');
       except
         on E:EFOpenError do ShowMessage('Try-Except-Finally' + #13 + #13 + 'Datei nicht gefunden!');
       end;
  end;  // case
end;

{ **************************************************************************** }

procedure TForm1.EdtZahlKeyPress(Sender: TObject; var Key: Char);
begin
  // if not (Key in ['0'..'9',#8]) then Key := #0;
end;

{ **************************************************************************** }

procedure TForm1.BitBtnInformationClick(Sender: TObject);
begin
  MessageDlg('Ausnahmebehandlung-Demo (V1.0)' + #13 + #13 +
    'Demonstrationsprogramm zur Ausnahmebehandlung' + #13 +
    'Try-Except-Finally in Delphi mit folgenden Exceptions:' + #13 + #13 +
    '- Division durch Null (EZeroDivide)' + #13 +
    '- Typkonvertierungsfehler bei StrToInt (EConvertError)' + #13 +
    '- Laden eines nicht existierenden Bildes (EFOpenError)' + #13 + #13 +
    'M. Zirbes, 2023' + #13 + #13 +
    'Bei diesem Programm handelt es sich um Freeware,' + #13 +
    'd.h. es steht zur kostenlosen Nutzung frei zur Verfgung' + #13 +
    'und darf ohne Entgelt vervielfltigt und verbreitet werden.' + #13 + #13 +
    'Kontakt: schule@mijozi.de',mtInformation,[mbOK],0);
end;

{ **************************************************************************** }

procedure TForm1.BitBtnSchliessenClick(Sender: TObject);
begin
  Close;
end;

{ **************************************************************************** }
{ **************************************************************************** }

end.

